"use strict";

// Load plugins
const autoprefixer = require("gulp-autoprefixer");
const browsersync = require("browser-sync").create();
const cleanCSS = require("gulp-clean-css");
const del = require("del");
const gulp = require("gulp");
const header = require("gulp-header");
const merge = require("merge-stream");
const plumber = require("gulp-plumber");
const rename = require("gulp-rename");
const sass = require("gulp-sass");
const babel = require("gulp-babel");
const uglify = require("gulp-uglify");
const concat = require("gulp-concat");

// Load package.json for banner
const pkg = require('./package.json');

// Set the banner content
const banner = ['/*!\n',
  ' * Start Bootstrap - <%= pkg.title %> v<%= pkg.version %> (<%= pkg.homepage %>)\n',
  ' * Copyright 2013-' + (new Date()).getFullYear(), ' <%= pkg.author %>\n',
  ' * Licensed under <%= pkg.license %> (https://github.com/BlackrockDigital/<%= pkg.name %>/blob/master/LICENSE)\n',
  ' */\n',
  '\n'
].join('');

// BrowserSync
function browserSync(done) {
  browsersync.init({
    server: {
      baseDir: "./"
    },
    port: 3000
  });
  done();
}

// BrowserSync reload
function browserSyncReload(done) {
  browsersync.reload();
  done();
}

// Clean vendor
function clean() {
  return del(["./output/"]);
}

function cleanTemplate() {
  return del(["../../college/src/main/resources/templates/"], {force: true});
}

// Bring third party dependencies from node_modules into vendor directory
function modules() {
  // Bootstrap JS
  var bootstrapJS = gulp.src('./node_modules/bootstrap/dist/js/*')
    .pipe(gulp.dest('./output/vendor/bootstrap/js'));
  // Bootstrap SCSS
  var bootstrapSCSS = gulp.src('./node_modules/bootstrap/scss/**/*')
    .pipe(gulp.dest('./public/vendor/bootstrap/scss'))
    .pipe(gulp.dest('./output/vendor/bootstrap/scss'));
  // ChartJS
  // var chartJS = gulp.src('./node_modules/chart.js/dist/*.js')
  //   .pipe(gulp.dest('./output/vendor/chart.js'));
  // dataTables
  // var dataTables = gulp.src([
  //     './node_modules/datatables.net/js/*.js',
  //     './node_modules/datatables.net-bs4/js/*.js',
  //     './node_modules/datatables.net-bs4/css/*.css'
  //   ])
  //   .pipe(gulp.dest('./output/vendor/datatables'));
  // Font Awesome
  var fontAwesome = gulp.src('./node_modules/@fortawesome/**/*')
    .pipe(gulp.dest('./output/vendor'));
  // jQuery Easing
  var jqueryEasing = gulp.src('./node_modules/jquery.easing/*.js')
    .pipe(gulp.dest('./output/vendor/jquery-easing'));
  // jQuery
  var jquery = gulp.src([
      './node_modules/jquery/dist/*',
      '!./node_modules/jquery/dist/core.js'
    ])
    .pipe(gulp.dest('./output/vendor/jquery'));
  var pagination = gulp.src([
    './public/vendor/bootstrap-paginator.js'
  ])
  .pipe(gulp.dest('./output/vendor'));

  var lodash = gulp.src([
    './public/vendor/lodash.js'
  ])
  .pipe(gulp.dest('./output/vendor'));
  var typehead = gulp.src([
    './node_modules/jquery-typeahead/dist/jquery.typeahead.min.js',
    './node_modules/jquery-typeahead/dist/jquery.typeahead.min.css'
  ])
  .pipe(gulp.dest('./public/vendor/jquery-typeahead'))
  .pipe(gulp.dest('./output/vendor/jquery-typeahead'));
  return merge(bootstrapJS, bootstrapSCSS, chartJS, dataTables, fontAwesome, jquery, jqueryEasing, pagination, typehead, lodash);
}

// CSS task
function scss() {
  return gulp
    .src("./public/scss/**/*.scss")
    .pipe(plumber())
    .pipe(sass({
      outputStyle: "expanded",
      includePaths: "./node_modules",
    }))
    .on("error", sass.logError)
    .pipe(autoprefixer({
      browsers: ['last 2 versions'],
      cascade: false
    }))
    .pipe(header(banner, {
      pkg: pkg
    }))
    .pipe(gulp.dest("./output/css"))
    .pipe(rename({
      suffix: ".min"
    }))
    .pipe(cleanCSS())
    .pipe(gulp.dest("./output/css"))
    .pipe(browsersync.stream());
}

function css() {
  return gulp
    .src([
      "./public/css/**/*.css",
      "./public/vendor/jquery-typehead/jquery.typeahead.min.css"
    ])
    .pipe(gulp.dest('./output/css'))
}

function jsVendor() {
  return gulp
    .src([
      './public/vendor/jquery/jquery.min.js',
      './public/vendor/bootstrap/js/bootstrap.bundle.min.js',
      './public/vendor/jquery-easing/jquery.easing.min.js',
      './public/vendor/lodash.js',
      './public/vendor/jquery-typehead/jquery.typeahead.min.js',
      './public/vendor/sb-admin-2.min.js',
    ])
    .pipe(concat('vendor.js'))
    .pipe(rename({
      suffix: '.min'
    }))
    .pipe(gulp.dest('./public/vendor'))
    .pipe(gulp.dest('./output/js/vendor'))
}

// JS task
function js() {
  return gulp
    .src([
      './public/js/**/*.js',
      '!./public/js/*.min.js',
      '!./public/js/bootstrap-paginator.js',
    ])
    .pipe(babel()) // bootstrap-paginatorb不能babel
    // .pipe(uglify())
    .pipe(header(banner, {
      pkg: pkg
    }))
    // .pipe(rename({
    //   suffix: '.min'
    // }))
    .pipe(gulp.dest('./output/js'))
}

function rawjs() {
  return gulp
    .src([
      './public/js/bootstrap-paginator.js',
    ])
    .pipe(gulp.dest('./output/js'))
}

function html() {
  return gulp
    .src([
      './public/**/*.html',
    ])
    .pipe(gulp.dest('./output/'))
}

function img() {
  return gulp
    .src([
      './public/img/**/*',
    ])
    .pipe(gulp.dest('./output/img'))
}

function deploy() {
  return gulp
    .src([
      './output/**/*'
    ])
    .pipe(gulp.dest('../../college/src/main/resources/templates/'))
    .pipe(gulp.dest('../../college/target/classes/templates/'))
}

// Watch files
function watchFiles() {
  gulp.watch("./public/scss/**/*", css);
  gulp.watch("./public/css/**/*", css);
  gulp.watch("./public/js/**/*", js);
  gulp.watch("./public/vendor/bootstrap-paginator.js", rawjs);
  gulp.watch("./public/**/*.html", html);
  gulp.watch("./output/**/*", deploy);
}

// Define complex tasks
// const vendor = gulp.series(clean, modules);
const build = gulp.series(clean, gulp.parallel(html, css, js, jsVendor, rawjs, img));
const watch = gulp.series(build, gulp.parallel(watchFiles/*, browserSync*/));
const publish = gulp.series(cleanTemplate, build, deploy)

// Export tasks
exports.css = css;
exports.js = js;
exports.clean = clean;
// exports.vendor = vendor;
exports.build = build;
exports.watch = watch;
exports.html = html;
exports.default = build;
exports.publish = publish;
// exports.deploy = deploy;
exports.jsVendor = jsVendor;
